% =========================================================================
% Solution to the MatLab excercise on 19.06.2017
% Task 2
%
% AUTHOR: Fabien Jenne, Peter Hofmeier
% Systems control and optimization laboratory, 
% Department of Microsystems Engineering (IMTEK),
% University of Freiburg
% 
% File Created: 13.06.2017
% =========================================================================
%
% For more information see the corresponding pdf
% 

clear;
clc;
close all;

%% (a) forward direction
disp('--- forward direction');

% Parameter definition
V = 5;  % m/s
L1 = 6; % m
L2 =4;  % m

% ODE for reference
% xdot = [V * cos(x(3));
%         V * sin(x(3));
%         V / L1 * tan(u);
%         V/L2*cos(x(3)-x(4)-pi/2)];

% x  = [ beta;
%        gamma ] = y
    
% steady state
x_ss = [0, 0];
u_ss = 0;
    
    
% linearized system matrices
% A = df/dx(x_ss, u_ss)     [2x2]
A = [ [ 0,                               0                           ]; ...
      [ V/L2 * cos(x_ss(1)-x_ss(2)),     -V/L2 * cos(x_ss(1)-x_ss(2))] ];

% B = df/du(x_ss, u_ss)     [2x1]
B = [ V/L1 * 1/(cos(u_ss)^2); ...
               0           ];

% C = dg/dx(x_ss, u_ss)     [1x2]
C = [  0,      1  ];

% D = dg/du(x_ss, u_ss)     [1x1]
D =  0 ;

%% (b) define the linearized state space system
sys_forward = ss(A, B, C, D);

%% (c) compute the eigenvalues and eigenvectors
[V1, D1] = eig(A);
disp('eigenvalues:')
disp(D1)
dimA = size(A, 1);
rankV = rank(V1);

% determine stability
stability_forward = isstable(sys_forward);
disp('stable?');
if stability_forward
    disp('> yes')
else
    % check for linear independence of the eigenvectors
    disp('> no')
    disp('marginal stability?')
    if rankV == dimA && ~any(diag(D1) > 0)
        disp('look at eigenvectors')
        disp(V1)
        disp('> yes')
    else
        disp('> no')
    end
end


%% (d) backward direction
disp('--- backward direction');


% adjust the velocity
V = -5;  % m/s
    
% steady state stays the same

% update the system matrices
% A = df/dx(x_ss, u_ss)     [4x4]
A2 = [ [ 0,                              0                           ]; ...
       [ V/L2 * cos(x_ss(1)-x_ss(2)),    -V/L2 * cos(x_ss(1)-x_ss(2))] ];

% B = df/du(x_ss, u_ss)     [4x1]
B2 = [ V/L1 * 1/(cos(u_ss)^2); ...
       0 ];

% C = dg/dx(x_ss, u_ss)     [4x4]
C2 = [  0,      1  ];

% D = dg/du(x_ss, u_ss)     [4x1]
D2 =  0 ;

% define the linearized state space system
sys_backward = ss(A2, B2, C2, D2);

% compute the eigenvalues and eigenvectors
[V2, D2] = eig(A2);
disp('eigenvalues:')
disp(D2)
dimA2 = size(A2, 1);
rankV2 = rank(V2);

% determine stability
stability_backward = isstable(sys_backward);
disp('stable?');
if stability_backward
    disp('> yes')
else
    disp('> no')
    disp('marginal stability?')
    if rankV2 == dimA2 && ~any(diag(D2) > 0)
        disp(V2)
        disp('> yes')
    else
        disp('> no')
    end
end

%% (e) simulation of the linearized systems

% simulation settings
stepSize = 0.01;
steps = 1000;
t = 0:stepSize:stepSize*(steps-1);
x_0 = [0; 0];
u = 0.1*ones(1,steps);

% forward direction
[~, t, x_sim] = lsim(sys_forward, u, t, x_0);
% backward direction
[~, t2, x2_sim] = lsim(sys_backward, u, t, x_0);

% angular difference
angle_diff_forward_const = x_sim(:,2)-x_sim(:,1);
angle_diff_backward_const = x2_sim(:,2)-x2_sim(:,1);


%% (f) plot the simulation results
figure();
hold on
plot(t, x_sim(:,1), '-r');
plot(t, x_sim(:,2), '-b');
title('linearized tractor with trailer model - forward');
xlabel('time / s');
ylabel('angle / rad');
legend({'$\beta$', '$\gamma$'}, 'Interpreter', 'latex', ...
    'Location', 'southeast');

figure();
hold on
plot(t2, x2_sim(:,1), '-r');
plot(t2, x2_sim(:,2), '-b');
title('linearized tractor with trailer model - backward');
xlabel('time / s');
ylabel('angle / rad');
legend({'$\beta$', '$\gamma$'}, 'Interpreter', 'latex', ...
    'Location', 'southeast');

%% (g) sinusiodial input

% creating the input signal
f=0.4;
u_sin = sin(2*pi*f*t);
u_sin(501:1000) = 0;

% simulating
% forward direction
[~, t, x_sim] = lsim(sys_forward, u_sin, t, x_0);
% backward direction
[~, t2, x2_sim] = lsim(sys_backward, u_sin, t, x_0);

% angular difference
angle_diff_forward_sin = x_sim(:,2)-x_sim(:,1);
angle_diff_backward_sin = x2_sim(:,2)-x2_sim(:,1);

figure();
subplot(2,1,1);
hold on
plot(t, x_sim(:,1), '-r');
plot(t, x_sim(:,2), '-b');
title('linearized tractor with trailer model - forward');
xlabel('time / s');
ylabel('angle / rad');
legend({'$\beta$', '$\gamma$'}, 'Interpreter', 'latex', ...
    'Location', 'southeast');

subplot(2,1,2);
plot(t, u_sin);
title('u');
xlabel('time / s');
ylabel('u(t)');

figure();
hold on
plot(t2, x2_sim(:,1), '-r');
plot(t2, x2_sim(:,2), '-b');
title('linearized tractor with trailer model - backward');
xlabel('time / s');
ylabel('angle / rad');
legend({'$\beta$', '$\gamma$'}, 'Interpreter', 'latex', ...
    'Location', 'southeast');

%% (h) animation for forward and backward motion with sinusiodial input

%animation forward
X=[zeros(size(x_sim)),x_sim]';
animation(X,u_sin,(t(2)-t(1)));

%animation backward
X=[zeros(size(x2_sim)),x2_sim]';
animation(X,u_sin,(t2(2)-t2(1)));

%% (i) plot the angular differences for different inputs and directions

figure();
subplot(2,1,1);
hold on;
plot(t, angle_diff_forward_const, '-r')
plot(t2, angle_diff_forward_sin, '-b');
title('angular difference between \beta and \gamma - forward');
xlabel('time / s');
ylabel('angle / rad');
legend({'constant input', 'sinusiodial'}, 'Location', 'southeast');

subplot(2,1,2);
hold on;
plot(t, angle_diff_backward_const, '-r')
plot(t2, angle_diff_backward_sin, '-b');
title('angular difference between \beta and \gamma - backward');
xlabel('time / s');
ylabel('angle / rad');
legend({'constant input', 'sinusiodial'}, 'Location', 'southeast');

