%==========================================================================
% The function defines the ODE of a tractor with the states X(the
% x-position), Y(the y-position) and beta(the orientation). The input is
% the steering angle alpha
% The nonlinear ODE is given by:
%   dX/dt = V * cos(beta)
%   dY/dt = V * sin(beta)
%   dbeta/dt = V/L * tan(alpha)
% 
% Input Parameter: state x, control input u; both are column vectors 
% Output Parameter: dx/dt; is a column vector
%
% AUTHOR: Joerg Fischer, Lukas Klar
% Systems control and optimization laboratory, 
% Department of Microsystems Engineering (IMTEK),
% University of Freiburg
%==========================================================================
function [ xdot ] = tractor_trailer_ode_v_neg(x, u)
    % Parameter definition
    V = -5; % m/s
    L1 = 6; % m
    L2 =4; % m
    % ODE
    xdot = [V * cos(x(3));
            V * sin(x(3));
            V / L1 * tan(u);
            V/L2*cos(x(3)-x(4)-pi/2)];
end